@@ -15,6 +15,18 @@ import adaptive_scheduler._mock_scheduler
 from adaptive_scheduler.utils import _progress, _RequireAttrsABCMeta
 
 
+def _run_submit(cmd, name=None):
+    returncode = None
+    env = os.environ.copy()
+    if name is not None:
+        env["NAME"] = name
+    while returncode != 0:
+        returncode = subprocess.run(
+            cmd.split(), stderr=subprocess.PIPE, env=env,
+        ).returncode
+        time.sleep(0.5)
+
+
 class BaseScheduler(metaclass=_RequireAttrsABCMeta):
     """Base object for a Scheduler.
 
@@ -45,6 +57,9 @@ class BaseScheduler(metaclass=_RequireAttrsABCMeta):
     extra_env_vars : list, optional
         Extra environment variables that are exported in the job
         script. e.g. ``["TMPDIR='/scratch'", "PYTHONPATH='my_dir:$PYTHONPATH'"]``.
+    extra_script : str, optional
+        Extra script that will be executed after any environment variables are set,
+        but before the main scheduler is run.
 
     Returns
     -------
@@ -64,6 +79,7 @@ class BaseScheduler(metaclass=_RequireAttrsABCMeta):
         num_threads,
         extra_scheduler,
         extra_env_vars,
+        extra_script,
     ):
         self.cores = cores
         self.run_script = run_script
@@ -74,6 +90,7 @@ class BaseScheduler(metaclass=_RequireAttrsABCMeta):
         self.num_threads = num_threads
         self._extra_scheduler = extra_scheduler
         self._extra_env_vars = extra_env_vars
+        self._extra_script = extra_script if extra_script is not None else ""
         self._JOB_ID_VARIABLE = "${JOB_ID}"
 
     @abc.abstractmethod
@@ -209,7 +226,7 @@ class BaseScheduler(metaclass=_RequireAttrsABCMeta):
             if self.cores <= 1:
                 raise ValueError(
                     "`ipyparalllel` uses 1 cores of the `adaptive.Runner` and"
-                    "the rest of the cores for the engines, so use more than 1 core."
+                    " the rest of the cores for the engines, so use more than 1 core."
                 )
             return self._ipyparallel(name)
         elif self.executor_type == "process-pool":
@@ -241,21 +258,21 @@ class BaseScheduler(metaclass=_RequireAttrsABCMeta):
         extra_env_vars = self._extra_env_vars or []
         return "\n".join(f"export {arg}" for arg in extra_env_vars)
 
+    @property
+    def extra_script(self):
+        """Script that will be run before the main scheduler."""
+        return str(self._extra_script) or ""
+
     def write_job_script(self, name: str) -> None:
         with open(self.batch_fname(name), "w") as f:
-            job_script = self.job_script(name)
+            job_script = self.job_script()
             f.write(job_script)
 
     def start_job(self, name: str) -> None:
         """Writes a job script and submits it to the scheduler."""
         self.write_job_script(name)
-        returncode = None
-        while returncode != 0:
-            returncode = subprocess.run(
-                f"{self.submit_cmd} {self.batch_fname(name)}".split(),
-                stderr=subprocess.PIPE,
-            ).returncode
-            time.sleep(0.5)
+        submit_cmd = f"{self.submit_cmd} {self.batch_fname(name)}"
+        _run_submit(submit_cmd)
 
     def __getstate__(self) -> dict:
         return dict(
@@ -286,6 +303,7 @@ class PBS(BaseScheduler):
         num_threads=1,
         extra_scheduler=None,
         extra_env_vars=None,
+        extra_script=None,
         *,
         cores_per_node=None,
     ):
@@ -299,6 +317,7 @@ class PBS(BaseScheduler):
             num_threads,
             extra_scheduler,
             extra_env_vars,
+            extra_script,
         )
         # Attributes that all schedulers need to have
         self._ext = ".batch"
@@ -367,7 +386,7 @@ class PBS(BaseScheduler):
         ]
         return [stdout, stderr]
 
-    def job_script(self, name: str) -> str:
+    def job_script(self) -> str:
         """Get a jobscript in string form.
 
         Returns
@@ -381,7 +400,6 @@ class PBS(BaseScheduler):
             #!/bin/sh
             #PBS -l nodes={self.nnodes}:ppn={self.cores_per_node}
             #PBS -V
-            #PBS -N {name}
             #PBS -o /tmp/placeholder
             #PBS -e /tmp/placeholder
             {{extra_scheduler}}
@@ -393,6 +411,8 @@ class PBS(BaseScheduler):
 
             cd $PBS_O_WORKDIR
 
+            {{extra_script}}
+
             {{executor_specific}}
             """
         )
@@ -400,12 +420,21 @@ class PBS(BaseScheduler):
         job_script = job_script.format(
             extra_scheduler=self.extra_scheduler,
             extra_env_vars=self.extra_env_vars,
-            executor_specific=self._executor_specific(name),
+            extra_script=self.extra_script,
+            executor_specific=self._executor_specific("${NAME}"),
             job_id_variable=self._JOB_ID_VARIABLE,
         )
 
         return job_script
 
+    def start_job(self, name: str) -> None:
+        """Writes a job script and submits it to the scheduler."""
+        name_prefix = name.rsplit("-", 1)[0]
+        self.write_job_script(name_prefix)
+        name_opt = f"-N {name}"
+        submit_cmd = f"{self.submit_cmd} {name_opt} {self.batch_fname(name_prefix)}"
+        _run_submit(submit_cmd, name)
+
     @staticmethod
     def _split_by_job(lines):
         jobs = [[]]
@@ -500,6 +529,7 @@ class SLURM(BaseScheduler):
         num_threads=1,
         extra_scheduler=None,
         extra_env_vars=None,
+        extra_script=None,
     ):
         super().__init__(
             cores,
@@ -511,6 +541,7 @@ class SLURM(BaseScheduler):
             num_threads,
             extra_scheduler,
             extra_env_vars,
+            extra_script,
         )
         # Attributes that all schedulers need to have
         self._ext = ".sbatch"
@@ -545,7 +576,7 @@ class SLURM(BaseScheduler):
             """
         )
 
-    def job_script(self, name: str) -> str:
+    def job_script(self) -> str:
         """Get a jobscript in string form.
 
         Returns
@@ -553,14 +584,11 @@ class SLURM(BaseScheduler):
         job_script : str
             A job script that can be submitted to SLURM.
         """
-        output_fname = self.output_fnames(name)[0].replace(self._JOB_ID_VARIABLE, "%A")
         job_script = textwrap.dedent(
             f"""\
             #!/bin/bash
-            #SBATCH --job-name {name}
             #SBATCH --ntasks {self.cores}
             #SBATCH --no-requeue
-            #SBATCH --output {output_fname}
             {{extra_scheduler}}
 
             export MKL_NUM_THREADS={self.num_threads}
@@ -568,6 +596,8 @@ class SLURM(BaseScheduler):
             export OMP_NUM_THREADS={self.num_threads}
             {{extra_env_vars}}
 
+            {{extra_script}}
+
             {{executor_specific}}
             """
         )
@@ -575,10 +605,24 @@ class SLURM(BaseScheduler):
         job_script = job_script.format(
             extra_scheduler=self.extra_scheduler,
             extra_env_vars=self.extra_env_vars,
-            executor_specific=self._executor_specific(name),
+            extra_script=self.extra_script,
+            executor_specific=self._executor_specific("${NAME}"),
         )
         return job_script
 
+    def start_job(self, name: str) -> None:
+        """Writes a job script and submits it to the scheduler."""
+        name_prefix = name.rsplit("-", 1)[0]
+        self.write_job_script(name_prefix)
+
+        output_fname = self.output_fnames(name)[0].replace(self._JOB_ID_VARIABLE, "%A")
+        output_opt = f"--output {output_fname}"
+        name_opt = f"--job-name {name}"
+        submit_cmd = (
+            f"{self.submit_cmd} {name_opt} {output_opt} {self.batch_fname(name_prefix)}"
+        )
+        _run_submit(submit_cmd, name)
+
     def queue(self, me_only: bool = True) -> Dict[str, Dict[str, str]]:
         python_format = {
             "jobid": 100,
@@ -642,6 +686,7 @@ class LocalMockScheduler(BaseScheduler):
         num_threads=1,
         extra_scheduler=None,
         extra_env_vars=None,
+        extra_script=None,
         *,
         mock_scheduler_kwargs=None,
     ):
@@ -656,6 +701,7 @@ class LocalMockScheduler(BaseScheduler):
             num_threads,
             extra_scheduler,
             extra_env_vars,
+            extra_script,
         )
         # LocalMockScheduler specific
         self.mock_scheduler_kwargs = mock_scheduler_kwargs or {}
@@ -677,7 +723,7 @@ class LocalMockScheduler(BaseScheduler):
             **super().__getstate__(), mock_scheduler_kwargs=self.mock_scheduler_kwargs
         )
 
-    def job_script(self, name: str) -> str:
+    def job_script(self) -> str:
         """Get a jobscript in string form.
 
         Returns
@@ -687,7 +733,7 @@ class LocalMockScheduler(BaseScheduler):
 
         Notes
         -----
-        Currenty there is a problem that this will not properly cleanup.
+        Currently, there is a problem that this will not properly cleanup.
         for example `ipengine ... &` will be detached and go on,
         normally a scheduler will take care of this.
         """
@@ -701,13 +747,16 @@ class LocalMockScheduler(BaseScheduler):
             export OMP_NUM_THREADS={self.num_threads}
             {{extra_env_vars}}
 
+            {{extra_script}}
+
             {{executor_specific}}
             """
         )
 
         job_script = job_script.format(
             extra_env_vars=self.extra_env_vars,
-            executor_specific=self._executor_specific(name),
+            executor_specific=self._executor_specific("${NAME}"),
+            extra_script=self.extra_script,
             job_id_variable=self._JOB_ID_VARIABLE,
         )
 
@@ -718,13 +767,8 @@ class LocalMockScheduler(BaseScheduler):
 
     def start_job(self, name: str) -> None:
         self.write_job_script(name)
-        returncode = None
-        while returncode != 0:
-            returncode = subprocess.run(
-                f"{self.submit_cmd} {name} {self.batch_fname(name)}".split(),
-                stderr=subprocess.PIPE,
-            ).returncode
-            time.sleep(0.5)
+        submit_cmd = f"{self.submit_cmd} {name} {self.batch_fname(name)}"
+        _run_submit(submit_cmd, name)
 
     @property
     def extra_scheduler(self):
